@@ -115,19 +115,19 @@ void FloatWin::Create()
 	button= gtk_button_new();
 	gtk_container_add(GTK_CONTAINER(button),gtk_image_new_from_stock(GTK_STOCK_FIND,GTK_ICON_SIZE_MENU));
 	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
-	g_signal_connect(G_OBJECT(button),"clicked", G_CALLBACK(on_query_click),NULL);
+	g_signal_connect(G_OBJECT(button),"clicked", G_CALLBACK(on_query_click), this);
 	g_signal_connect(G_OBJECT(button),"enter_notify_event", G_CALLBACK(stardict_on_enter_notify), NULL);
 	gtk_box_pack_start(GTK_BOX(button_hbox),button,false,false,0);
 	gtk_tooltips_set_tip(gpAppFrame->tooltips,button,_("Query in the main window"),NULL);
 
 	button= gtk_button_new();
-	gtk_container_add(GTK_CONTAINER(button),gtk_image_new_from_stock(GTK_STOCK_COPY,GTK_ICON_SIZE_MENU));
+	gtk_container_add(GTK_CONTAINER(button),gtk_image_new_from_stock(GTK_STOCK_SAVE,GTK_ICON_SIZE_MENU));
 	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
 	g_signal_connect(G_OBJECT(button),"clicked", 
-			 G_CALLBACK(on_copy_click), this);
+			 G_CALLBACK(on_save_click), this);
 	g_signal_connect(G_OBJECT(button),"enter_notify_event", G_CALLBACK(stardict_on_enter_notify), NULL);
 	gtk_box_pack_start(GTK_BOX(button_hbox),button,false,false,0);
-	gtk_tooltips_set_tip(gpAppFrame->tooltips,button,_("Copy"),NULL);
+	gtk_tooltips_set_tip(gpAppFrame->tooltips,button,_("Save to file"),NULL);
 
 	PronounceWordButton= gtk_button_new();
 	gtk_container_add(GTK_CONTAINER(PronounceWordButton),gtk_image_new_from_stock(GTK_STOCK_EXECUTE,GTK_ICON_SIZE_MENU));
@@ -142,18 +142,19 @@ void FloatWin::Create()
 	StopButton= gtk_button_new();
 	gtk_container_add(GTK_CONTAINER(StopButton),gtk_image_new_from_stock(GTK_STOCK_STOP,GTK_ICON_SIZE_MENU));
 	gtk_button_set_relief (GTK_BUTTON (StopButton), GTK_RELIEF_NONE);
-	g_signal_connect(G_OBJECT(StopButton),"clicked", G_CALLBACK(on_stop_click),NULL);
+	g_signal_connect(G_OBJECT(StopButton),"clicked", G_CALLBACK(on_stop_click), this);
 	g_signal_connect(G_OBJECT(StopButton),"enter_notify_event", G_CALLBACK(stardict_on_enter_notify), NULL);
 	gtk_box_pack_start(GTK_BOX(button_hbox),StopButton,false,false,0);
-	gtk_tooltips_set_tip(gpAppFrame->tooltips, StopButton, _("Stop selection scanning"),NULL);
+	gtk_tooltips_set_tip(gpAppFrame->tooltips, StopButton, _("Stop selection-scanning"),NULL);
 
 	gtk_widget_set_sensitive(gpAppFrame->oFloatWin.StopButton, 
 													 conf->get_bool("/apps/stardict/preferences/dictionary/scan_selection"));
 
+#ifndef CONFIG_GPE
 	button= gtk_button_new();
 	gtk_container_add(GTK_CONTAINER(button),gtk_image_new_from_stock(GTK_STOCK_HELP,GTK_ICON_SIZE_MENU));
 	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
-	g_signal_connect(G_OBJECT(button),"clicked", G_CALLBACK(on_help_click),NULL);
+	g_signal_connect(G_OBJECT(button),"clicked", G_CALLBACK(on_help_click), this);
 	g_signal_connect(G_OBJECT(button),"enter_notify_event", G_CALLBACK(stardict_on_enter_notify), NULL);
 	gtk_box_pack_start(GTK_BOX(button_hbox),button,false,false,0);
 	gtk_tooltips_set_tip(gpAppFrame->tooltips,button,_("Help"),NULL);
@@ -161,10 +162,11 @@ void FloatWin::Create()
 	button= gtk_button_new();
 	gtk_container_add(GTK_CONTAINER(button),gtk_image_new_from_stock(GTK_STOCK_QUIT,GTK_ICON_SIZE_MENU));
 	gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
-	g_signal_connect(G_OBJECT(button),"clicked", G_CALLBACK(on_quit_click),NULL);
+	g_signal_connect(G_OBJECT(button),"clicked", G_CALLBACK(on_quit_click), this);
 	g_signal_connect(G_OBJECT(button),"enter_notify_event", G_CALLBACK(stardict_on_enter_notify), NULL);
 	gtk_box_pack_start(GTK_BOX(button_hbox),button,false,false,0);
 	gtk_tooltips_set_tip(gpAppFrame->tooltips,button,_("Quit"),NULL);
+#endif
 
 	button = gtk_button_new();
 
@@ -184,55 +186,74 @@ void FloatWin::Create()
 	gtk_widget_hide(button_hbox); //show all will show hbox's children,now hide hbox only.	
 }
 
-void FloatWin::ShowText(const gchar **word, gchar **data,const gchar * sOriginWord)
+void FloatWin::ShowText(gchar ***Word, gchar ****WordData, const gchar * sOriginWord)
 {
-  QueryingWord = sOriginWord;
-  found_result = FLOAT_WIN_FOUND;
-  view->BeginUpdate();
-  view->Clear();
-  std::string mark = "<b><span size=\"x-large\">";
-  gchar *m_str = g_markup_escape_text(sOriginWord, -1);
-  mark += m_str;
-  g_free(m_str);
-  mark += "</span></b>";
-  view->AppendPangoText(mark.c_str());
-
-  for (int i=0; i<gpAppFrame->oLibs.ndicts(); i++) {
-    if (word[i])
-      view->AppendHeader(gpAppFrame->oLibs.dict_name(i), word[i],
-												 strcmp(word[i], sOriginWord)==0);
-    
-    if (data[i])
-      view->AppendData(data[i]);
-  }
-  view->EndUpdate();
-
-  gboolean canRead = gpAppFrame->oReadWord.canRead(sOriginWord);
-  if (canRead) {
-    PronounceWord = sOriginWord;
-  } else {
-    for (int i=0;i< gpAppFrame->oLibs.ndicts(); i++) {
-      if (word[i] && strcmp(word[i], sOriginWord)) {
-	if (gpAppFrame->oReadWord.canRead(word[i])) {
-	  canRead = TRUE;
-	  PronounceWord = word[i];
+	QueryingWord = sOriginWord;
+	found_result = FLOAT_WIN_FOUND;
+	view->BeginUpdate();
+	view->Clear();
+	std::string mark = "<b><span size=\"x-large\">";
+	gchar *m_str = g_markup_escape_text(sOriginWord, -1);
+	mark += m_str;
+	g_free(m_str);
+	mark += "</span></b>";
+	view->AppendPangoText(mark.c_str());
+	int j,k;
+	for (int i=0; i<gpAppFrame->oLibs.ndicts(); i++) {
+		if (Word[i]) {
+			view->AppendNewline();
+			view->AppendHeader(gpAppFrame->oLibs.dict_name(i), i);
+			j=0;
+			do {
+				if (j==0) {
+					if (strcmp(Word[i][j], sOriginWord))
+						view->AppendWord(Word[i][j]);
+				} else {
+					view->AppendNewline();
+					view->AppendWord(Word[i][j]);
+				}
+				view->AppendData(WordData[i][j][0], Word[i][j]);
+				k=1;
+				while (WordData[i][j][k]) {
+					view->AppendNewline();
+					view->AppendDataSeparate();
+					view->AppendData(WordData[i][j][k], Word[i][j]);
+					k++;
+				}
+				j++;
+			} while (Word[i][j]);
+		}
 	}
-	break;
-      }
-    }
-  }
-  gtk_widget_set_sensitive(PronounceWordButton, canRead);
+	view->EndUpdate();
+	gboolean pronounced = false;
+	gboolean canRead = gpAppFrame->oReadWord.canRead(sOriginWord);
+	if (canRead) {
+		if (PronounceWord == sOriginWord)
+			pronounced = true;
+		else
+			PronounceWord = sOriginWord;
+	} else {
+		for (int i=0;i< gpAppFrame->oLibs.ndicts(); i++) {
+			if (Word[i] && strcmp(Word[i][0], sOriginWord)) {
+				if (gpAppFrame->oReadWord.canRead(Word[i][0])) {
+					canRead = TRUE;
+					if (PronounceWord == Word[i][0])
+						pronounced = true;
+					else
+						PronounceWord = Word[i][0];
+				}
+				break;
+			}
+		}
+	}
+	gtk_widget_set_sensitive(PronounceWordButton, canRead);
 
-  Popup(true);
-  bool pronounce_when_popup=
-		conf->get_bool("/apps/stardict/preferences/floating_window/pronounce_when_popup");
-  if (canRead && pronounce_when_popup)
-    gpAppFrame->oReadWord.read(PronounceWord.c_str());
+	Popup(true);
+	if (canRead && (!pronounced) && conf->get_bool("/apps/stardict/preferences/floating_window/pronounce_when_popup"))
+		gpAppFrame->oReadWord.read(PronounceWord.c_str());
 }
 
-void FloatWin::ShowText(gchar ***pppWord, gchar ***pppWordData, 
-												const gchar ** ppOriginWord, gint count,
-												const gchar *sOriginWord)
+void FloatWin::ShowText(gchar ****ppppWord, gchar *****pppppWordData, const gchar ** ppOriginWord, gint count, const gchar *sOriginWord)
 {
   QueryingWord = sOriginWord;
   found_result = FLOAT_WIN_FUZZY_FOUND;
@@ -275,7 +296,10 @@ void FloatWin::ShowText(gchar ***pppWord, gchar ***pppWordData,
   mark += "</span>";
   view->AppendPangoText(mark.c_str());
 
+  int m,n;
   for (j=0; j<count; j++) {
+    if (!ppppWord[j])
+	continue;
     mark = "\n\n<b><span size=\"x-large\">";
     m_str = g_markup_escape_text(ppOriginWord[j],-1);
     mark += m_str;
@@ -283,14 +307,31 @@ void FloatWin::ShowText(gchar ***pppWord, gchar ***pppWordData,
     mark += "</span></b>";
     view->AppendPangoText(mark.c_str());
     for (int i=0; i<gpAppFrame->oLibs.ndicts(); i++) {
-      if (pppWord[j][i])
-				view->AppendHeader(gpAppFrame->oLibs.dict_name(i), pppWord[j][i],
-													 strcmp(ppOriginWord[j],pppWord[j][i])==0);
-      
-			if (pppWordData[j][i])
-				view->AppendData(pppWordData[j][i]);      
+	if (ppppWord[j][i]) {
+		view->AppendNewline();
+		view->AppendHeader(gpAppFrame->oLibs.dict_name(i), i);
+		m=0;
+		do {
+			if (m==0) {
+				if (strcmp(ppppWord[j][i][m], ppOriginWord[j]))
+					view->AppendWord(ppppWord[j][i][m]);
+			} else {
+				view->AppendNewline();
+				view->AppendWord(ppppWord[j][i][m]);
+			}
+			view->AppendData(pppppWordData[j][i][m][0], ppppWord[j][i][m]);
+			n=1;
+			while (pppppWordData[j][i][m][n]) {
+				view->AppendNewline();
+				view->AppendDataSeparate();
+				view->AppendData(pppppWordData[j][i][m][n], ppppWord[j][i][m]);
+				n++;
+			}
+			m++;
+		} while (ppppWord[j][i][m]);
+	}
     }
-  }		
+  }	
   view->EndUpdate();
 
   gboolean canRead = gpAppFrame->oReadWord.canRead(sOriginWord);
@@ -300,11 +341,11 @@ void FloatWin::ShowText(gchar ***pppWord, gchar ***pppWordData,
 
  
   Popup(false);
-  bool pronounce_when_popup=
+  /*bool pronounce_when_popup=
 		conf->get_bool("/apps/stardict/preferences/floating_window/pronounce_when_popup");
 
   if (canRead && pronounce_when_popup)
-    gpAppFrame->oReadWord.read(PronounceWord.c_str());
+    gpAppFrame->oReadWord.read(PronounceWord.c_str());*/
 }
 
 void FloatWin::ShowNotFound(const char* sWord,const char* sReason, gboolean fuzzy)
@@ -314,6 +355,10 @@ void FloatWin::ShowNotFound(const char* sWord,const char* sReason, gboolean fuzz
 		found_result = FLOAT_WIN_FUZZY_NOT_FOUND;
 	else
 		found_result = FLOAT_WIN_NOT_FOUND;
+
+	if (!conf->get_bool("/apps/stardict/preferences/floating_window/show_if_not_found"))
+		return;
+
 	gchar *text;
 	gchar *m_word,*m_reason;
 	m_word = g_markup_escape_text(sWord,-1);
@@ -321,9 +366,14 @@ void FloatWin::ShowNotFound(const char* sWord,const char* sReason, gboolean fuzz
 	text = g_strdup_printf("<b><big>%s</big></b>\n<span foreground=\"blue\">%s</span>",m_word,m_reason);
 	view->SetPangoText(text);
 	
+	gboolean pronounced = false;
 	gboolean canRead = gpAppFrame->oReadWord.canRead(sWord);
-	if (canRead)
-		PronounceWord = sWord;
+	if (canRead) {
+		if (PronounceWord == sWord)
+			pronounced = true;
+		else
+			PronounceWord = sWord;
+	}
 	gtk_widget_set_sensitive(PronounceWordButton, canRead);
 
 	if (fuzzy)
@@ -334,9 +384,7 @@ void FloatWin::ShowNotFound(const char* sWord,const char* sReason, gboolean fuzz
 	g_free(m_word);
 	g_free(m_reason);
 
-	bool pronounce_when_popup=
-		conf->get_bool("/apps/stardict/preferences/floating_window/pronounce_when_popup");
-	if (canRead && pronounce_when_popup)
+	if (canRead && (!pronounced) && conf->get_bool("/apps/stardict/preferences/floating_window/pronounce_when_popup"))
 		gpAppFrame->oReadWord.read(PronounceWord.c_str());
 }
 
@@ -457,6 +505,7 @@ void FloatWin::Hide()
 		timeout = 0;
 	}
 	button_box_once_shown = false;
+	PronounceWord.clear();
 	gtk_widget_hide(FloatWindow);
 }
 
@@ -550,8 +599,12 @@ gboolean FloatWin::vEnterNotifyCallback (GtkWidget *widget, GdkEventCrossing *ev
 			g_print("GDK_NOTIFY_INFERIOR\n");
 			break;
 	}*/
-	
+
+#ifdef _WIN32
+	if ((event->detail==GDK_NOTIFY_ANCESTOR) || (event->detail==GDK_NOTIFY_NONLINEAR) || (event->detail==GDK_NOTIFY_NONLINEAR_VIRTUAL)) {
+#else
 	if ((event->detail==GDK_NOTIFY_NONLINEAR) || (event->detail==GDK_NOTIFY_NONLINEAR_VIRTUAL)) {
+#endif
 		if (!GTK_WIDGET_VISIBLE(oFloatWin->button_hbox)) {
 			gtk_widget_show(oFloatWin->button_hbox);
 		
@@ -622,7 +675,11 @@ gboolean FloatWin::vLeaveNotifyCallback (GtkWidget *widget, GdkEventCrossing *ev
 			g_print("GDK_NOTIFY_INFERIOR\n");
 			break;
 	}*/
+#ifdef _WIN32
+	if (((event->detail==GDK_NOTIFY_ANCESTOR) || (event->detail==GDK_NOTIFY_NONLINEAR) || (event->detail==GDK_NOTIFY_NONLINEAR_VIRTUAL))&&(!oFloatWin->ismoving)) {
+#else
 	if (((event->detail==GDK_NOTIFY_NONLINEAR) || (event->detail==GDK_NOTIFY_NONLINEAR_VIRTUAL))&&(!oFloatWin->ismoving)) {
+#endif
 		gtk_widget_hide(oFloatWin->button_hbox);
 	}
 	return true;
@@ -630,12 +687,18 @@ gboolean FloatWin::vLeaveNotifyCallback (GtkWidget *widget, GdkEventCrossing *ev
 
 void FloatWin::on_menu_copy_activate(GtkWidget * widget, FloatWin *oFloatWin)
 {
-	oFloatWin->on_copy_click(widget, NULL);
+	GtkClipboard* clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
+	gtk_clipboard_set_text(clipboard, oFloatWin->view->GetText().c_str(), -1);
+}
+
+void FloatWin::on_menu_save_activate(GtkWidget * widget, FloatWin *oFloatWin)
+{
+	oFloatWin->on_save_click(widget, oFloatWin);
 }
 
 void FloatWin::on_menu_query_activate(GtkWidget * widget, FloatWin *oFloatWin)
 {
-	oFloatWin->on_query_click(widget, NULL);
+	oFloatWin->on_query_click(widget, oFloatWin);
 }
 
 void FloatWin::on_menu_play_activate(GtkWidget * widget, FloatWin *oFloatWin)
@@ -670,7 +733,11 @@ gboolean FloatWin::vButtonPressCallback (GtkWidget * widget, GdkEventButton * ev
 			gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), image);
 			g_signal_connect(G_OBJECT(menuitem), "activate", G_CALLBACK(on_menu_copy_activate), oFloatWin);
 			gtk_menu_shell_append(GTK_MENU_SHELL(oFloatWin->menu), menuitem);
-
+			menuitem = gtk_image_menu_item_new_with_mnemonic(_("_Save"));
+			image = gtk_image_new_from_stock(GTK_STOCK_SAVE, GTK_ICON_SIZE_MENU);
+			gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), image);
+			 g_signal_connect(G_OBJECT(menuitem), "activate", G_CALLBACK(on_menu_save_activate), oFloatWin);
+			gtk_menu_shell_append(GTK_MENU_SHELL(oFloatWin->menu), menuitem);
 			menuitem = gtk_image_menu_item_new_with_mnemonic(_("_Query"));
 			image = gtk_image_new_from_stock(GTK_STOCK_FIND, GTK_ICON_SIZE_MENU);
 			gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), image);
@@ -736,13 +803,13 @@ gboolean FloatWin::vMotionNotifyCallback (GtkWidget * widget, GdkEventMotion * e
 	return true;
 }
 
-void FloatWin::on_query_click(GtkWidget *widget, gpointer data)
+void FloatWin::on_query_click(GtkWidget *widget, FloatWin *oFloatWin)
 {
 	play_sound_on_event("buttonactive");
 
 	if (!conf->get_bool("/apps/stardict/preferences/floating_window/lock"))
-		gpAppFrame->oFloatWin.Hide();
-	gpAppFrame->Query(gpAppFrame->oFloatWin.QueryingWord.c_str());	
+		oFloatWin->Hide();
+	gpAppFrame->Query(oFloatWin->QueryingWord.c_str());	
 #ifdef _WIN32
 	if (!GTK_WIDGET_VISIBLE(gpAppFrame->window))
 		gpAppFrame->oDockLet.stardict_systray_maximize(gpAppFrame->window);
@@ -750,12 +817,24 @@ void FloatWin::on_query_click(GtkWidget *widget, gpointer data)
 	gtk_window_present(GTK_WINDOW(gpAppFrame->window));
 }
 
-void FloatWin::on_copy_click(GtkWidget *widget, FloatWin *oFloatWin)
+void FloatWin::on_save_click(GtkWidget *widget, FloatWin *oFloatWin)
 {
+	if (conf->get_bool("/apps/stardict/preferences/dictionary/only_export_word")) {
+		FILE *fp = fopen(conf->get_string("/apps/stardict/preferences/dictionary/export_file").c_str(), "a+");
+		if(fp) {
+			fputs(oFloatWin->QueryingWord.c_str(),fp);
+			fputs("\n",fp);
+			fclose(fp);
+		}
+	} else {
+		FILE *fp = fopen(conf->get_string("/apps/stardict/preferences/dictionary/export_file").c_str(), "a+");
+		if(fp) {
+			fputs(oFloatWin->view->GetText().c_str(),fp);
+			fputs("\n\n",fp);
+			fclose(fp);
+		}
+	}
 	play_sound_on_event("buttonactive");
-	
-  GtkClipboard* clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
-  gtk_clipboard_set_text(clipboard, oFloatWin->view->GetText().c_str(), -1);
 }
 
 void FloatWin::on_play_click(GtkWidget *widget, FloatWin *oFloatWin)
@@ -763,26 +842,28 @@ void FloatWin::on_play_click(GtkWidget *widget, FloatWin *oFloatWin)
 	gpAppFrame->oReadWord.read(oFloatWin->PronounceWord.c_str());
 }
 
-void FloatWin::on_stop_click(GtkWidget *widget, gpointer data)
+void FloatWin::on_stop_click(GtkWidget *widget, FloatWin *oFloatWin)
 {
 	play_sound_on_event("buttonactive");
 	conf->set_bool("/apps/stardict/preferences/dictionary/scan_selection", false);
 }
 
-void FloatWin::on_help_click(GtkWidget *widget, gpointer data)
+#ifndef CONFIG_GPE
+void FloatWin::on_help_click(GtkWidget *widget, FloatWin *oFloatWin)
 {
 	play_sound_on_event("buttonactive");
 
 	if (!conf->get_bool("/apps/stardict/preferences/floating_window/lock"))
-		gpAppFrame->oFloatWin.Hide();
+		oFloatWin->Hide();
 	show_help("stardict-scan-selection");
 }
 
-void FloatWin::on_quit_click(GtkWidget *widget, gpointer data)
+void FloatWin::on_quit_click(GtkWidget *widget, FloatWin *oFloatWin)
 {
 	play_sound_on_event("buttonactive");
 	gpAppFrame->Quit();
 }
+#endif
 
 void FloatWin::vLockCallback(GtkWidget *widget, FloatWin *oFloatWin)
 {
