@@ -2,14 +2,17 @@
 #define __SD_READWORD_H__
 
 #include <glib.h>
+#include <list>
+#include <string>
 
 class ReadWord {
 public:
 	ReadWord();
-  bool canRead(const gchar *word);
+	void loadpath(const gchar *path);
+	bool canRead(const gchar *word);
 	void read(const gchar *word);
 private:
-  bool have_data_file;
+	std::list<std::string> ttspath;
 };
 
 #endif
